import json
from .models.issue import Issue

def save_issues_to_json(issues: list[Issue], file_path: str) -> None:
    issues_data = [issue.to_dict() for issue in issues]

    with open(file_path, 'w') as f:
        json.dump(issues_data, f, indent=4)


import requests
from urllib.parse import quote
from lxml import etree

def _get_delpher_item_identifiers_from_ppn(ppn_number: int, constraints: str = "(date >= 1940) AND (date <= 1945)", collection: str = 'DDD_artikel', max_records: int = 25):
    identifiers = []
    query = quote(f"{constraints} AND ppn any ({ppn_number})")
    start_record = 1

    while True:
        url = f"https://jsru.kb.nl/sru/sru?operation=searchRetrieve&version=1.2&recordSchema=dc&query={query}&x-collection={collection}&maximumRecords={max_records}&startRecord={start_record}"
    
        response = requests.get(url)
        
        if response.status_code != 200:
                raise Exception(f"SRU request failed: {response.status_code}")
        
        content = response.content
        root = etree.fromstring(content)
        ns = {
            "srw": "http://www.loc.gov/zing/srw/",
            "dc": "http://purl.org/dc/elements/1.1/"
        }

        nr_of_records = int(root.xpath("srw:numberOfRecords/text()", namespaces=ns)[0])
        batch_identifiers = root.xpath("//srw:recordData/dc:identifier/text()", namespaces=ns)
        identifiers.extend(batch_identifiers)


        if start_record + max_records > nr_of_records:
            break
        start_record += max_records
    return identifiers

def get_issue_identifiers(ppn_number: int, constraints: str = "(date >= 1940) AND (date <= 1945)", collection: str = 'DDD_artikel', max_records: int = 25):
    delpher_item_identifiers = _get_delpher_item_identifiers_from_ppn(ppn_number, constraints, collection, max_records)
    issue_identifiers = set()
    for item in delpher_item_identifiers:
        issue_id = item.split("urn=")[1].split(':a0')[0]
        issue_identifiers.add(issue_id)
    return list(issue_identifiers)


from typing import Optional

class Article:
    def __init__(
        self,
        identifier: str,
        title: str,
        subject: Optional[str] = None,
        url: Optional[str] = None,
        ocr: Optional[str] = None,
        ocr_text: Optional[str] = None
    ):
        self.identifier = identifier
        self.title = title
        self.subject = subject
        self.url = url
        self.ocr = ocr
        self.ocr_text = ocr_text

    def to_dict(self):
        return {
            'identifier': self.identifier,
            'title': self.title,
            'subject': self.subject,
            'url': self.url,
            'ocr': self.ocr,
            'ocr_text': self.ocr_text
        }


def load_ppn_numbers_from_txt(file_path: str) -> list[str]:
    ppn_numbers = []
    with open(file_path, encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            ppn_numbers.append(line)
    return ppn_numbers    


from sickle import Sickle

def oai_get_record(identifier: str, prefix: str = 'DDD:'):
    oai_identifier = prefix + identifier
    sickle = Sickle('https://services.kb.nl/mdo/oai')
    record = sickle.GetRecord(identifier=oai_identifier, metadataPrefix='didl')
    return record


from typing import Optional
from .article import Article

class Page:
    def __init__(
        self,
        identifier: str,
        page_number: Optional[int] = None,
        image: Optional[str] = None,
        url: Optional[str] = None,
        articles: Optional[list[Article]] = None,
    ):
        self.identifier = identifier
        self.page_number = page_number
        self.image = image
        self.url = url
        self.articles = articles

    def to_dict(self):
        return {
            'identifier': self.identifier,
            'page_number': self.page_number,
            'image': self.image,
            'url': self.url,
            'article_identifiers': [article.identifier for article in self.articles]
        }